@@ -1,20 +1,19 @@
-#!/usr/local/bin/python3.13
+#!/usr/pkg/bin/python3.5
 # -*- python -*-
 
 # Keep this script in sync with python-config.sh.in
 
 import getopt
+import os
 import sys
 import sysconfig
 
 valid_opts = ['prefix', 'exec-prefix', 'includes', 'libs', 'cflags',
-              'ldflags', 'extension-suffix', 'help', 'abiflags', 'configdir',
-              'embed']
+              'ldflags', 'extension-suffix', 'help', 'abiflags', 'configdir']
 
 def exit_with_usage(code=1):
     print("Usage: {0} [{1}]".format(
-        sys.argv[0], '|'.join('--'+opt for opt in valid_opts)),
-        file=sys.stdout if code == 0 else sys.stderr)
+        sys.argv[0], '|'.join('--'+opt for opt in valid_opts)), file=sys.stderr)
     sys.exit(code)
 
 try:
@@ -25,8 +24,8 @@ except getopt.error:
 if not opts:
     exit_with_usage()
 
+pyver = sysconfig.get_config_var('VERSION')
 getvar = sysconfig.get_config_var
-pyver = getvar('VERSION')
 
 opt_flags = [flag for (flag, val) in opts]
 
@@ -35,10 +34,10 @@ if '--help' in opt_flags:
 
 for opt in opt_flags:
     if opt == '--prefix':
-        print(getvar('prefix'))
+        print(sysconfig.get_config_var('prefix'))
 
     elif opt == '--exec-prefix':
-        print(getvar('exec_prefix'))
+        print(sysconfig.get_config_var('exec_prefix'))
 
     elif opt in ('--includes', '--cflags'):
         flags = ['-I' + sysconfig.get_path('include'),
@@ -48,27 +47,23 @@ for opt in opt_flags:
         print(' '.join(flags))
 
     elif opt in ('--libs', '--ldflags'):
-        libs = []
-        if '--embed' in opt_flags:
-            libs.append('-lpython' + pyver + sys.abiflags)
-        else:
-            libpython = getvar('LIBPYTHON')
-            if libpython:
-                libs.append(libpython)
-        libs.extend(getvar('LIBS').split() + getvar('SYSLIBS').split())
-
+        libs = ['-lpython' + pyver + sys.abiflags]
+        libs += getvar('LIBS').split()
+        libs += getvar('SYSLIBS').split()
         # add the prefix/lib/pythonX.Y/config dir, but only if there is no
         # shared library in prefix/lib/.
         if opt == '--ldflags':
             if not getvar('Py_ENABLE_SHARED'):
                 libs.insert(0, '-L' + getvar('LIBPL'))
+            if not getvar('PYTHONFRAMEWORK'):
+                libs.extend(getvar('LINKFORSHARED').split())
         print(' '.join(libs))
 
     elif opt == '--extension-suffix':
-        print(getvar('EXT_SUFFIX'))
+        print(sysconfig.get_config_var('EXT_SUFFIX'))
 
     elif opt == '--abiflags':
         print(sys.abiflags)
 
     elif opt == '--configdir':
-        print(getvar('LIBPL'))
+        print(sysconfig.get_config_var('LIBPL'))
