@@ -1,4 +1,4 @@
-#!/usr/bin/env python3
+#!/usr/pkg/bin/python3.5
 
 """
   ----------------------------------------------
@@ -89,12 +89,13 @@ import sys
 import os
 
 from tkinter import *
-from idlelib.colorizer import ColorDelegator, color_config
-from idlelib.percolator import Percolator
-from idlelib.textview import view_text
+from idlelib.ColorDelegator import ColorDelegator, color_config
+from idlelib.Percolator import Percolator
+from idlelib.textView import view_text
 from turtledemo import __doc__ as about_turtledemo
 
 import turtle
+import time
 
 demo_dir = os.path.dirname(os.path.abspath(__file__))
 darwin = sys.platform == 'darwin'
@@ -136,7 +137,7 @@ class DemoWindow(object):
             import subprocess
             # Make sure we are the currently activated OS X application
             # so that our menu bar appears.
-            subprocess.run(
+            p = subprocess.Popen(
                     [
                         'osascript',
                         '-e', 'tell application "System Events"',
